""" 自然言語理解APIの雑談ドメインを拡張するサンプル
https://developer.yahoo.co.jp/webapi/jlp/nlu/v2/doc/response.html
"""

import os
import sys
from abc import ABCMeta, abstractmethod

import yj.nlu.api


class Generator(metaclass=ABCMeta):
    @abstractmethod
    def match(self, nlu_result):
        pass

    @abstractmethod
    def generate_response(self, nlu_result):
        pass


class ChatGenerator(Generator):
    RESPONSES = {
        "AGENT_AGE": "年齢は秘密です。",
        "AGENT_BIRTHDATE": "誕生日は秘密です。",
        "GREETINGS_BYE": "また話しかけてくださいね。",
        "GREETINGS_GOOD_MORNING": "おはようございます。お話できて嬉しいです。",
    }

    def match(self, nlu_result):
        "自然言語理解APIが推定した、発話の意図"
        domain = nlu_result["METHOD"]
        " 発話の意図が雑談 (SAY) の時のみTrueを返す "
        return domain == "SAY"

    def generate_response(self, nlu_result):
        if "PARAM_SMALL_TALK_INTENT" in nlu_result:
            small_talk_intent = nlu_result["PARAM_SMALL_TALK_INTENT"]
            if small_talk_intent in ChatGenerator.RESPONSES:
                response = ChatGenerator.RESPONSES[small_talk_intent]
                return response

        if "PARAM_TEXT" not in nlu_result:
            raise ValueError("PARAM_TEXT not found")
        response = nlu_result["PARAM_TEXT"]

        return response


class Assistant(object):
    def __init__(self, nlu):
        self.nlu = nlu
        self.generator = ChatGenerator()

    def talk(self, utterance):
        response = self.nlu.analyze(utterance)
        result = response["result"]
        if self.generator.match(result):
            return self.generator.generate_response(result)
        return "すみません。よくわかりませんでした。"


def main():
    nlu_appid = os.getenv("NLU_API_APPID", "your NLU appid")
    nlu = yj.nlu.api.NluApi(appid=nlu_appid)
    assistant = Assistant(nlu)

    utterance = sys.stdin.readline().rstrip()
    while utterance:
        sys.stdout.write(f"入力:{utterance}\t")

        response = assistant.talk(utterance)
        sys.stdout.write(f"応答:{response}\n")

        utterance = sys.stdin.readline().rstrip()


if __name__ == "__main__":
    main()
