import os
import sys
from abc import ABCMeta, abstractmethod

import yj.nlu.api


class Generator(metaclass=ABCMeta):
    @abstractmethod
    def match(self, nlu_result):
        pass

    @abstractmethod
    def generate_response(self, nlu_result):
        pass


class CustomGenerator(Generator):
    old2new = {
        "睦月": "1月",
        "如月": "2月",
        "弥生": "3月",
        "卯月": "4月",
        "皐月": "5月",
        "水無月": "6月",
        "文月": "7月",
        "葉月": "8月",
        "長月": "9月",
        "神無月": "10月",
        "霜月": "11月",
        "師走": "12月",
    }
    new2old = {
        "1月": "睦月",
        "2月": "如月",
        "3月": "弥生",
        "4月": "卯月",
        "5月": "皐月",
        "6月": "水無月",
        "7月": "文月",
        "8月": "葉月",
        "9月": "長月",
        "10月": "神無月",
        "11月": "霜月",
        "12月": "師走",
    }

    def match(self, nlu_result):
        # 自然言語理解APIが推定した発話の意図
        domain = nlu_result["METHOD"]
        # 発話の意図がskill.jsonで定義したものならTrueを返す
        return (domain == "ask_old_calendar") or (domain == "ask_calendar")

    def generate_response(self, nlu_result):
        month_from = nlu_result["PARAM_MONTH"]
        domain = nlu_result["METHOD"]
        if domain == "ask_old_calendar":
            month_to = CustomGenerator.new2old[month_from]
            return f"{month_from}の旧暦は{month_to}です"
        if domain == "ask_calendar":
            month_to = CustomGenerator.old2new[month_from]
            return f"{month_from}は{month_to}です"

        raise ValueError(f"invalid domain: {domain}")


class Assistant(object):
    def __init__(self, nlu):
        self.nlu = nlu
        self.generator = CustomGenerator()

    def talk(self, utterance):
        response = self.nlu.analyze(utterance)
        result = response["result"]
        if self.generator.match(result):
            return self.generator.generate_response(result)
        return "すみません。よくわかりませんでした。"


def main():
    appid = os.getenv("NLU_API_APPID", "your appid")

    nlu = yj.nlu.api.NluApi(appid=appid)
    # カスタマイズしたいドメインなどの情報を反映させる
    nlu.load_skill("skill.json")

    assistant = Assistant(nlu)

    utterance = sys.stdin.readline().rstrip()
    while utterance:
        sys.stdout.write(f"入力:{utterance}\t")
        response = assistant.talk(utterance)
        sys.stdout.write(f"応答:{response}\n")

        utterance = sys.stdin.readline().rstrip()


if __name__ == "__main__":
    main()
