""" 自然言語理解APIの天気ドメインを利用するサンプル
https://developer.yahoo.co.jp/webapi/jlp/nlu/v2/doc/response.html
"""

import os
import sys
from abc import ABCMeta, abstractmethod

import yj.nlu.api


class WeatherAPI(object):
    """天気予報APIのmock"""

    def get(self, params):
        if params["city"] == "東京":
            return {"city": params["city"], "temperature": 30, "weather": "晴れ"}
        elif params["city"] == "大阪":
            return {"city": params["city"], "temperature": 25, "weather": "雨"}
        else:
            return {"city": params["city"], "temperature": 28, "weather": "曇り"}


class Generator(metaclass=ABCMeta):
    @abstractmethod
    def match(self, nlu_result):
        pass

    @abstractmethod
    def generate_response(self, nlu_result):
        pass


class WeatherInformationGenerator(Generator):
    def __init__(self):
        self.api = WeatherAPI()

    def match(self, nlu_result):
        # 自然言語理解APIが推定した発話の意図
        domain = nlu_result["METHOD"]
        # 発話の意図が天気情報の取得の場合のみTrueを返す
        return domain == "WEATHER"

    def generate_response(self, nlu_result):
        if "PARAM_PLACE" not in nlu_result:
            return "場所を指定して言い直してください。"
        city = nlu_result["PARAM_PLACE"]

        if "PARAM_METHOD_SUBCAT" not in nlu_result:
            raise ValueError("PARAM_METHOD_SUBCAT not found")
        subdomain = nlu_result["PARAM_METHOD_SUBCAT"]

        weather_result = self.api.get({"city": city})

        if subdomain == "TEMPERATURE":
            # 天気予報APIから返却された気温の情報を使って応答を生成する
            temperature = weather_result["temperature"]
            return f"{city}の気温は{temperature}度です。"

        # それ以外は天気の情報を使って応答を生成する
        weather = weather_result["weather"]
        return f"{city}の天気は{weather}です。"


class Assistant(object):
    def __init__(self, nlu):
        self.nlu = nlu
        self.generator = WeatherInformationGenerator()

    def talk(self, utterance):
        response = self.nlu.analyze(utterance)
        result = response["result"]
        if self.generator.match(result):
            return self.generator.generate_response(result)
        return "すみません。よくわかりませんでした。"


def main():
    nlu_appid = os.getenv("NLU_API_APPID", "your NLU appid")

    nlu = yj.nlu.api.NluApi(appid=nlu_appid)
    assistant = Assistant(nlu)

    utterance = sys.stdin.readline().rstrip()
    while utterance:
        sys.stdout.write(f"入力:{utterance}\t")
        response = assistant.talk(utterance)
        sys.stdout.write(f"応答:{response}\n")

        utterance = sys.stdin.readline().rstrip()


if __name__ == "__main__":
    main()
