""" yj.nlu.api module """

import json

import requests

import yj.nlu.exceptions
import yj.nlu.utils


class NluApi(object):
    """
    An interface class for YJDN NLU API

    Attributes
    ----------
    headers : dict
        HTTP request headers
    rules : dict of {str: str}
        Custom rules which customize responses of NLU API
    dictionaries : dict of {str: str}
        User-defined dictionaries used in custom rules
    """

    #: Endpoint of NLU API
    URL = "https://jlp.yahooapis.jp/NLUService/V2/analyze"

    #: A deliminator used in custom rules and custom dictionaries.
    DELIM = ","

    #: Used as a prefix character of user-defined dictionary name.
    DICT_PREFIX = "$"

    #: Limit byte size of each query string value
    MAX_BYTES = 1023

    def __init__(self, appid):
        """
        Initialize NluApi with `headers`, `rules` and `dictionaries` objects.

        Parameters
        ----------
        appid : str
            Your application ID of YJDN NLU API
        """
        self.headers = {
            "Content-Type": "application/json",
            "User-Agent": "Yahoo AppID: {0}".format(appid),
        }
        self.rules = {}
        self.dictionaries = {}

    def _post(self, params):
        """
        Sends a POST request to NLU API.

        Parameters
        ----------
        params : dict of {str: str}
            Query strings to be sent.

        Raises
        ------
        NluApiError
            Raised when NLU API returns status code but 200.

        Returns
        -------
        response : requests.Response
        """
        response = requests.post(NluApi.URL, json.dumps(params), headers=self.headers)
        if response.status_code != 200:
            response_json = response.json()
            if "Message" in response_json:
                # WIP : handle YJDN NLU API error
                message = response_json["Message"]
            else:
                message = response_json["Error"]["Message"]
            raise yj.nlu.exceptions.NluApiError(response.status_code, message)
        return response

    def analyze(self, intext, context={}, **kwargs):
        """
        Sends a POST request to NLU API, gets response from NLU API and adds context information to the response.

        Parameters
        ----------
        intext : str
            An query string text to NLU API.
        context : dict of {str: str}, default {}
            Context information of previous requests.
            If this parameter is send to NLU API, the API refers context information and change its response.
            Use results["context"] as an input of `context`.
        **kwargs: dict of {str: str}
            Optional parameters that is sent to NLU API.

        Raises
        ------
        TypeError
            Raised when type of intext is not str

        Returns
        -------
        results : dict of {str: dict of {str: str}}
            Response of NLU API and context information.
            result : key-value pairs of NLU API response.
            context : key-value pairs of context information that are used for next requests.
        """
        if not isinstance(intext, str):
            raise TypeError("intext must be str")
        params = {"q": intext, "context": {}}
        params["context"].update(context)
        params["context"].update(self.dictionaries)
        params["context"].update(self.rules)
        params["context"].update(kwargs)
        request_body = {
            "jsonrpc": "2.0",
            "id": "YJDN_V2_SAMPLE",
            "method": "jlp.nluservice.analyze",
            "params": params,
        }

        response = self._post(request_body)
        results = response.json()
        results["context"] = self._prepare_next_context(results["result"], context)
        return results

    def load_skill(self, filename):
        """
        Loads custom rules to `rules` and dictionaries to `dictionaries` from ``.json`` file.

        Parameters
        ----------
        filename : str
            Input file path (``.json``).

        Raises
        ------
        IOError
            Raised when the input file does not exist.

        """
        with open(filename) as f:
            skill = json.load(f)
            if "dictionaries" in skill:
                for name, entities in skill["dictionaries"].items():
                    self.add_dictionary(name, entities)
            if "rules" in skill:
                for domain, patterns in skill["rules"].items():
                    for pattern in patterns:
                        self.add_rule(domain, pattern)

    def add_rule(self, domain, pattern):
        """
        Adds an custom rule to `rules`.

        Parameters
        ----------
        domain : str
            Domain of the rule.
        pattern : str
            Regex-like pattern of the rule.
        """
        key = "sample{0}".format(len(self.rules))
        rule = "{0}{1}{2}".format(domain, NluApi.DELIM, pattern)
        self.rules[key] = rule

    def add_dictionary(self, name, entities):
        """
        Adds an user-defined dictionary to `dictionaries`.

        Parameters
        ----------
        name : str
            Name of the dictionary.
        entities : list of str
            List of dictionary entities.

        Raises
        ------
        ValueError
            Raised when length of `name` or a entity in `entities` exceeds the limit.
            Also raised when `entities` is empty.
        """
        if len(entities) == 0:
            raise ValueError("No entity found")
        key = "dict{0}".format(len(self.dictionaries))

        num_bytes_for_name = yj.nlu.utils.count_bytes(NluApi.DICT_PREFIX + name)
        if num_bytes_for_name > NluApi.MAX_BYTES - 2:
            # no entity can be added in this condition
            raise ValueError("Length of the {}... is too long".format(name[:10]))
        num_bytes = num_bytes_for_name
        dictionary = NluApi.DICT_PREFIX + name
        for entity in entities:
            # +1 is a byte for deliminator
            num_byte = yj.nlu.utils.count_bytes(entity) + 1
            if num_byte + num_bytes_for_name > NluApi.MAX_BYTES:
                raise ValueError("Length of the {}... is too long".format(entity[:10]))
            if num_bytes + num_byte > NluApi.MAX_BYTES:
                self.dictionaries[key] = dictionary
                num_bytes = num_bytes_for_name + num_byte
                key = "dict{0}".format(len(self.dictionaries))
                dictionary = NluApi.DICT_PREFIX + name
                continue
            num_bytes += num_byte
            dictionary += NluApi.DELIM + entity
        self.dictionaries[key] = dictionary

    def clear_dictionary(self):
        """
        Remove all items from `dictionaries`.
        """
        self.dictionaries = {}

    def clear_rules(self):
        """
        Remove all items from `rules`.
        """
        self.rules = {}

    def _prepare_next_context(self, result, context):
        """
        Generate context information from response of NLU API and previous context information.

        Parameters
        ----------
        result : dict of {str: str}
            results["result"] of ``analyze()``
        context : dict of {str: str}
            results["context"] of ``analyze()``

        Returns
        -------
        next_context : dict of {str: str}
        """
        next_context = {}
        for k, v in context.items():
            if k.endswith("_PREV3"):
                continue
            if k.endswith("_PREV2"):
                key = k[:-1] + "3"
                next_context[key] = v
                continue
            if k.endswith("_PREV1"):
                key = k[:-1] + "2"
                next_context[key] = v
                continue
            key = k + "_PREV1"
            next_context[key] = v
        for k, v in result.items():
            key = k + "_PREV1"
            next_context[key] = v
        return next_context
